package com.example.ellegadodepintia.exploradoresDePintia.model

import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.widget.Button
import android.widget.GridLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.core.content.res.ResourcesCompat
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.Resumen
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context

object GestorDeEventos{

    private var puedeHacerClick = true

    fun gestionar() {
            val opcionesLayout = context!!.findViewById<GridLayout>(R.id.opciones)
            opcionesLayout.removeAllViews()

            GameState.eventoActual.opciones.forEachIndexed { index, opcion ->
                val boton = crearBotonDeOpcion(opcion, index)
                opcionesLayout.addView(boton)
            }
    }


    private fun crearBotonDeOpcion(opcion: Opcion, index: Int): Button {
        val boton = Button(context).apply {
            text = opcion.descripcion
            background = ContextCompat.getDrawable(context, R.drawable.style_boton_option)
            stateListAnimator = null
            typeface = ResourcesCompat.getFont(context, R.font.pixeled)

            isAllCaps = false
            setTextColor(ContextCompat.getColor(context, android.R.color.black))

            val atributoDrawable = when (opcion.atributoRequerida) {
                Atributo.Vida -> ContextCompat.getDrawable(context, R.drawable.icon_vida)
                Atributo.Ataque -> ContextCompat.getDrawable(context, R.drawable.icon_espada)
                Atributo.Investigacion -> ContextCompat.getDrawable(context, R.drawable.icon_vasija)
                Atributo.Energia -> ContextCompat.getDrawable(context, R.drawable.icon_energia)
                else -> null
            }

            atributoDrawable?.let {
                val width = (25 * resources.displayMetrics.density).toInt()
                val height = it.intrinsicHeight * width / it.intrinsicWidth
                it.setBounds(0, 0, width, height)
                setCompoundDrawables(it, null, null, null)
            }

            setOnClickListener {
                if (puedeHacerClick) {
                    puedeHacerClick = false

                    mostrarEfecto(opcion.aplicarEfecto())

                    Handler(Looper.getMainLooper()).postDelayed({
                        puedeHacerClick = true
                    }, 500)
                }
            }
        }

        val layoutParams = LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.MATCH_PARENT,
            LinearLayout.LayoutParams.WRAP_CONTENT
        ).apply {
            if ((index == 0 || index == 1) && GameState.eventoActual.opciones.size == 4) {
                marginEnd = (14 * context!!.resources.displayMetrics.density).toInt()
            }
            if (index == 1 || index == 3) {
                topMargin = (18 * context!!.resources.displayMetrics.density).toInt()
            }
            width = (165 * context!!.resources.displayMetrics.density).toInt()
            height = (50 * context!!.resources.displayMetrics.density).toInt()
        }

        boton.layoutParams = layoutParams

        return boton
    }

    fun mostrarOpcionElegirCamino() {
        val opcionesLayout = context!!.findViewById<GridLayout>(R.id.opciones)
        opcionesLayout.removeAllViews()

        val boton = Button(context).apply {
            "Seguir explorando".also { text = it }
            background = ContextCompat.getDrawable(context, R.drawable.style_boton_option)
            stateListAnimator = null
            typeface = ResourcesCompat.getFont(context, R.font.pixeled)

            isAllCaps = false
            setTextColor(ContextCompat.getColor(context, android.R.color.black))

            setOnClickListener {
                if(puedeHacerClick){
                    puedeHacerClick = false
                    GameState.jugador.eliminarTodosLosBuffos()
                    if (GeneradorDeEventos.comprobarEventoDisponible()){
                        GeneradorDeEventos.generarEvento(0)
                        gestionar()
                    } else {
                        val intent = Intent(context, Resumen::class.java)
                        context.startActivity(intent)
                    }

                    Handler(Looper.getMainLooper()).postDelayed({
                        puedeHacerClick = true
                    }, 1500)
                }
            }

        }
        opcionesLayout.addView(boton)

    }

    private fun mostrarEfecto(efecto: String) {
        context!!.findViewById<TextView>(R.id.textoResultado).text = efecto
    }


}